@@ -35,7 +35,7 @@ def opt_arguments(func):
 
 
 @opt_arguments
-def fetch_all(func, return_all=None, always_all=False):
+def fetch_all(func, return_all=None, always_all=False, pagination='anchor'):
     """
     Class method decorator for fetching all items. Add parameter `all=False` for decored method.
     If `all` is True, method runs as many times as it returns any results.
@@ -63,17 +63,20 @@ def fetch_all(func, return_all=None, always_all=False):
             if isinstance(instances, QuerySet):
                 if not instances_all:
                     instances_all = QuerySet().none()
-                instances_all |= instances
+                instances_count = instances.count()
+                if instances_count:
+                    instances_all |= instances
             elif isinstance(instances, list):
                 if not instances_all:
                     instances_all = []
+                instances_count = len(instances)
                 instances_all += instances
             else:
                 raise ValueError("Wrong type of response from func %s. It should be QuerySet or list, not a %s" % (func, type(instances)))
 
-            # resursive pagination
-            if 'has_more' in response and response['has_more'] or 'has_more' not in response and 'anchor' in response:
-                kwargs['anchor'] = response['anchor']
+            # recursive pagination
+            if instances_count and ('has_more' in response and response['has_more'] or 'has_more' not in response and pagination in response):
+                kwargs[pagination] = response.get(pagination)
                 return wrapper(self, all=all, instances_all=instances_all, **kwargs)
 
             if return_all:
@@ -144,7 +147,7 @@ def fetch_by_chunks_of(func, items_limit, ids_argument='ids'):
         ids = kwargs[ids_argument]
         if ids:
             kwargs_sliced = dict(kwargs)
-            for chunk in list_chunks_iterator(ids, self.fetch_users_limit):
+            for chunk in list_chunks_iterator(ids, items_limit):
                 kwargs_sliced[ids_argument] = chunk
                 instances = func(self, **kwargs_sliced)
 
