@@ -11,9 +11,11 @@ from odnoklassniki_api.utils import api_call#, OdnoklassnikiError
 from odnoklassniki_api import fields
 from odnoklassniki_api.decorators import atomic
 from fields_api import API_REQUEST_FIELDS
+from pytz import timezone, utc
 import logging
 import re
 
+
 log = logging.getLogger('odnoklassniki_api')
 
 COMMIT_REMOTE = getattr(settings, 'ODNOKLASSNIKI_API_COMMIT_REMOTE', True)
@@ -55,45 +57,40 @@ class OdnoklassnikiManager(models.Manager):
 
         super(OdnoklassnikiManager, self).__init__(*args, **kwargs)
 
-#     def get_by_url(self, url):
-#         '''
-#         Return vkonakte object by url
-#         '''
-#         m = re.findall(r'^(?:https?://)?vk.com/([^/]+)/?$', url)
-#         if not len(m):
-#             raise ValueError("Url should be started with http://vk.com/")
-#
-#         return self.get_by_slug(m[0])
-#
-#     def get_by_slug(self, slug):
-#         '''
-#         Return existed User, Group, Application by slug or new intance with empty pk
-#         '''
-#         try:
-#             assert self.model.slug_prefix and slug.startswith(self.model.slug_prefix)
-#             remote_id = int(re.findall(r'^%s(\d+)$' % self.model.slug_prefix, slug)[0])
-#         except (AssertionError, ValueError, IndexError):
-#             try:
-#                 response = api_call('resolveScreenName', **{'screen_name': slug})
-#                 assert self.model.resolve_screen_name_type == response['type']
-#                 remote_id = int(response['object_id'])
-#             except OdnoklassnikiError, e:
-#                 log.error("Method get_by_slug returned error instead of response. Slug: '%s'. Error: %s" % (slug, e))
-#                 return None
-#             except (KeyError, TypeError, ValueError), e:
-#                 log.error("Method get_by_slug returned response in strange format: %s. Slug is '%s'" % (response, slug))
-#                 return None
-#             except AssertionError:
-#                 log.error("Method get_by_slug returned instance with wrong type '%s', not '%s'. Slug is '%s'" % (response['type'], self.model.resolve_screen_name_type, slug))
-#                 return None
-#
-#         try:
-#             object = self.model.objects.get(remote_id=remote_id)
-#             object.screen_name = slug
-#         except self.model.DoesNotExist:
-#             object = self.model(remote_id=remote_id, screen_name=slug)
-#
-#         return object
+    def get_by_url(self, url):
+        '''
+        Return existed User, Group, Application by url or new intance with empty pk
+        '''
+        m = re.findall(r'^(?:https?://)?(?:www.)?(?:ok.ru|odnoklassniki.ru)/(.+)/?$', url)
+        if not len(m):
+            raise ValueError("Wrong domain: %s" % url)
+
+        slug = m[0]
+
+        try:
+            assert self.model.slug_prefix and slug.startswith(self.model.slug_prefix)
+            id = int(re.findall(r'^%s(\d+)$' % self.model.slug_prefix, slug)[0])
+        except (AssertionError, ValueError, IndexError):
+            try:
+                response = api_call('url.getInfo', url=url)
+                assert self.model.resolve_screen_name_type == response['type']
+                id = int(response['objectId'])
+            except OdnoklassnikiError, e:
+                log.error("Method get_by_slug returned error instead of response. URL='%s'. Error: %s" % (url, e))
+                return None
+            except (KeyError, TypeError, ValueError), e:
+                log.error("Method get_by_slug returned response in strange format: %s. URL='%s'" % (response, url))
+                return None
+            except AssertionError:
+                log.error("Method get_by_slug returned instance with wrong type '%s', not '%s'. URL='%s'" % (response['type'], self.model.resolve_screen_name_type, url))
+                return None
+
+        try:
+            object = self.model.objects.get(id=id)
+        except self.model.DoesNotExist:
+            object = self.model(id=id)#, shortname=slug)
+
+        return object
 
     def get_or_create_from_instances_list(self, instances):
         # python 2.6 compatibility
@@ -160,7 +157,7 @@ class OdnoklassnikiManager(models.Manager):
         TODO: rename everywhere extra_fields to _extra_fields
         '''
         extra_fields = kwargs.pop('extra_fields', {})
-        extra_fields['fetched'] = datetime.now()
+        extra_fields['fetched'] = datetime.utcnow().replace(tzinfo=utc)
 
         response = self.api_call(*args, **kwargs)
 
@@ -269,6 +266,7 @@ class OdnoklassnikiModel(models.Model):
     remote_pk_local_field = 'id'
     methods_access_tag = ''
     methods_namespace = ''
+    slug_prefix = ''
 
     fetched = models.DateTimeField(u'Обновлено', null=True, blank=True, db_index=True)
 
@@ -323,15 +321,15 @@ class OdnoklassnikiModel(models.Model):
                         pass
 
             elif isinstance(field, models.DateTimeField):
-                if isinstance(value, string_types) and len(value) == 19:
+
+
+                if isinstance(value, string_types) and len(value) >= 16:
                     try:
-                        value = datetime(int(value[0:4]), int(value[5:7]), int(value[8:10]), int(value[11:13]), int(value[14:16]), int(value[17:19]))
-                        assert value.year != 1970
-                    except:
-                        value = None
-                elif isinstance(value, string_types) and len(value) == 16:
-                    try:
-                        value = datetime(int(value[0:4]), int(value[5:7]), int(value[8:10]), int(value[11:13]), int(value[14:16]))
+                        if len(value) == 19:
+                            value = datetime(int(value[0:4]), int(value[5:7]), int(value[8:10]), int(value[11:13]), int(value[14:16]), int(value[17:19]))
+                        elif len(value) == 16:
+                            value = datetime(int(value[0:4]), int(value[5:7]), int(value[8:10]), int(value[11:13]), int(value[14:16]))
+                        value = timezone('Europe/Moscow').localize(value)
                         assert value.year != 1970
                     except:
                         value = None
@@ -339,7 +337,7 @@ class OdnoklassnikiModel(models.Model):
                     try:
                         value = int(value)
                         assert value > 0
-                        value = datetime.fromtimestamp(value)
+                        value = datetime.utcfromtimestamp(value).replace(tzinfo=utc)
                     except:
                         value = None
             elif isinstance(field, models.DateField):
@@ -369,10 +367,15 @@ class OdnoklassnikiModel(models.Model):
         Refresh current model with remote data
         """
         objects = self.__class__.remote.fetch(**self.refresh_kwargs)
-        if len(objects) == 1:
-            self.__dict__.update(objects[0].__dict__)
-        else:
-            raise OdnoklassnikiContentError("Remote server returned more objects, than expected - %d instead of one. Object details: %s, request details: %s" % (len(objects), self.__dict__, kwargs))
+        if isinstance(objects, models.Model):
+            new_instance = objects
+        elif isinstance(objects, (list, tuple, QuerySet)):
+            if len(objects) == 1:
+                new_instance = objects[0]
+            else:
+                raise OdnoklassnikiContentError("Remote server returned more objects, than expected - %d instead of one. Object details: %s, request details: %s" % (len(objects), self.__dict__, kwargs))
+
+        self.__dict__.update(new_instance.__dict__)
 
     def get_url(self):
         return 'http://odnoklassniki.ru/%s' % self.slug
